from time import sleep

import can

from resh.protocols.can_utils.rapida2 import commands
from resh.protocols.can_utils.rapida2.firmware_update_handler import FirmwareUpdateHandler
from resh.protocols.can_utils.rapida_packet import RapidaSend
from resh.utils import settings
from resh.utils.errors import CustomError, DeviceNotAnswered


class Rapida2:

    @classmethod
    def update_firmware(cls, module_class, path, module_address):
        """
        Обновление прошивки для модулей второй рапиды
        :param module_class: экземпляр класса update_firmware, нужен, т.к. пока не переписывал can senders
        :param path: Путь до прошивки
        :param module_address: Адрес прошиваемого модуля.
        """

        def _get_notifire():
            firmware_update_handler = FirmwareUpdateHandler(path, module_address)
            return can.Notifier(bus=firmware_update_handler.bus, listeners=[firmware_update_handler])

        module_class.module_type = 0
        module_class.dump_answer = True

        # Проверяем, находится ли модуль в состоянии загрузчика, если нет, перезагружаем
        try:
            cls._send_to_module_bootloader(0x01, module_address)
            notifire = _get_notifire()
        except DeviceNotAnswered:
            module_class.send_cmd(commands.R2QueryVersion)
            notifire = _get_notifire()
            module_class.send_cmd(commands.R2Reboot)

        # Даем две минуты на перепрошивку.
        for _ in range(24):
            sleep(5)
            _status = notifire.listeners[0].status
            # Проверяем, что процесс прошивки начался, т.к. на одной из версий был баг перезагрузки
            if _status == 0:
                module_class.send_cmd(commands.R2Reboot)
            elif _status in (8, 9):
                break
        else:
            notifire.stop()
            try:
                cls._send_to_module_bootloader(0x1E, module_address)
            except BaseException:
                pass
            raise CustomError('Firmware updating canceled by timeout')

        notifire.stop()
        if notifire.listeners[0].status == 9:
            raise CustomError(f'Firmware updating canceled with error: {notifire.listeners[0].exception_text}')

    @classmethod
    def _send_to_module_bootloader(cls, params: int, module_address, data_bytes: bytes = bytes()):
        try:
            if params > 31:
                raise Exception()
            packet = RapidaSend(0, module_address, settings.conf.protocol == 2, data_bytes)
            packet.arbitration_id_ext_bits = (59 << 6) + (params << 0)
            return packet.send(True)
        except Exception as e:
            raise e
