from .can_packet import CanSend, CanDump
from .rapida2 import commands as r2cmds
from .rapida2 import responses as r2resps
from ..payloads.relay_payload import RelayCommand, RelayServiceData
from ...utils import tools
from ...utils.logger import log


class RelaySend(CanSend):
    def __init__(self, module_type, module_address, end_frame, cmd=0, **kwargs):
        super(RelaySend, self).__init__(module_type, module_address, True, end_frame, cmd, **kwargs)
        self._arbitration_id_ext_bits = 0

    def check_commit(self, dump):
        return True

    def on_response(self, dump):
        if isinstance(dump, RelayDump):
            log.info('   <- %s' % dump.payload.command)
            self.stop_answer_timer()
            return dump.payload.command

    @property
    def committed(self):
        return True

    @property
    def arbitration_id_ext_bits(self) -> int:
        return self._arbitration_id_ext_bits

    @arbitration_id_ext_bits.setter
    def arbitration_id_ext_bits(self, value):
        self._arbitration_id_ext_bits = value

    @property
    def frame(self):
        return self.end_frame


class RelayDump(CanDump):
    @property
    def data(self) -> bytearray:
        return self.frame

    @property
    def opcode(self) -> int:
        if self.cmd:
            return tools.get_bits(self.arbitration_id_ext_bits, 6, 11)
        else:
            return tools.get_bits(self.arbitration_id_ext_bits, 9, 11)

    @property
    def params(self) -> int:
        if self.cmd:
            return tools.get_bits(self.arbitration_id_ext_bits, 0, 5)
        else:
            return tools.get_bits(self.arbitration_id_ext_bits, 0, 8)

    @property
    def payload(self):
        if isinstance(self, RelayDump):
            if self.cmd:
                _resp = r2resps.get_response(self.opcode, self.params, self.data)
            else:
                _resp = r2resps.get_data_response(self.opcode, self.params, self.data)

            if (_resp.__class__ == r2resps.R2RelayResponse) or (_resp.__class__ == r2resps.R2RelayDataTransfer):
                _cmd = r2cmds.get_command(self.opcode, self.data)
                if self.cmd:
                    return RelayCommand(self, _cmd)
                else:
                    return RelayServiceData(self, _cmd)
            else:
                return RelayServiceData(self, _resp)
        else:
            return None
