import unittest
from unittest import TestCase

import can

from resh.protocols.can_utils.can_packet import CanPacket, CanDump
from resh.protocols.can_utils.dali_packet import DaliDump, DaliSend
from resh.protocols.can_utils.rapida_packet import RapidaDump, ModuleDump
from resh.protocols.can_utils.relay_packet import RelayDump
from resh.protocols.payloads.dali import DaliCommand, DaliResponse
from resh.protocols.payloads.rapida import ServicePacket
from resh.protocols.payloads.relay_payload import RelayCommand, RelayServiceData


class TestCanExtPacket(TestCase):
    packet = CanPacket(from_id=31, to_id=1, module_type=1, ext_mode=True, end_frame=0x900B, cmd=0xf)

    def test_arbitration_id_ext(self):
        packet = CanPacket(from_id=31, to_id=1, module_type=1, ext_mode=True, end_frame=0x900B, cmd=0xf)
        self.assertEqual(packet.arbitration_id, 0xC202000)

    def test_arbitration_id(self):
        packet = CanPacket(from_id=31, to_id=1, module_type=1, ext_mode=False, end_frame=0x900B, cmd=0xf)
        self.assertEqual(packet.arbitration_id, 0x3E1)

    def test_module_desc(self):
        self.assertEqual(self.packet.module_desc, "Восьмиканальный модуль DALI")

    def test_module_name(self):
        self.assertEqual(self.packet.module_name, "RPMDA")

    def test_can_dump0(self):
        _msg = can.Message(arbitration_id=0x3E1, is_extended_id=False, data=[0x01, 0x04, 0x01, 0x0B, 0x90],
                           extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, DaliSend), True)
        self.assertEqual(_res.ext_mode, False)
        self.assertEqual(_res.frame, bytes(b'\x01\x04\x01\x0B\x90'))
        self.assertEqual(isinstance(_res.payload, DaliCommand), True)

    def test_can_dump1(self):
        _msg = can.Message(arbitration_id=0x03F, is_extended_id=False, data=[0x01, 0x08, 0x0B, 0xB8], extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, DaliDump), True)
        self.assertEqual(_res.ext_mode, False)
        self.assertEqual(_res.frame, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(isinstance(_res.payload, DaliCommand), True)

    def test_can_dump2(self):
        _msg = can.Message(arbitration_id=0xC202000, is_extended_id=True, data=[0x90, 0x0B], extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, DaliDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.frame, bytes(b'\x90\x0B'))
        self.assertEqual(isinstance(_res.payload, DaliCommand), True)

    def test_can_dump3(self):
        _msg = can.Message(arbitration_id=0x82036C2, is_extended_id=True, data=[0x90, 0x0B, 0xC2], extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, RapidaDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.arbitration_id_ext_bits, 0x6C2)
        self.assertEqual(_res.frame, bytes(b'\x90\x0B\xC2'))
        self.assertEqual(_res.data, bytes(b'\x90\x0B\xC2'))
        self.assertEqual(_res.command_byte, 0x6)
        _payload = _res.payload
        self.assertEqual(isinstance(_payload, ServicePacket), True)
        self.assertEqual(_payload.destination_str, '1')
        self.assertEqual(_payload.name, 'R2Response 90 0B C2')

    def test_can_dump4(self):
        _msg = can.Message(arbitration_id=0x82090C2, is_extended_id=True, data=[0x01, 0x08, 0x0B, 0xB8],
                           extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, RelayDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.arbitration_id_ext_bits, 0xC2)
        self.assertEqual(_res.frame, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(_res.data, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(_res.opcode, 0x3)
        self.assertEqual(_res.params, 0x2)
        _payload = _res.payload
        self.assertEqual(isinstance(_payload, RelayCommand), True)
        self.assertEqual(_payload.destination_str, '1')
        self.assertEqual(_payload.name, '1 R2RelaySetGroupState 4')  # + tools.bytes2str(_res.data).upper())

    def test_can_dump5(self):
        _msg = can.Message(arbitration_id=0x82086C2, is_extended_id=True, data=[0x01, 0x08, 0x0B, 0xB8],
                           extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, RelayDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.arbitration_id_ext_bits, 0x6C2)
        self.assertEqual(_res.frame, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(_res.data, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(_res.opcode, 0x3)
        self.assertEqual(_res.params, 0xC2)
        _payload = _res.payload
        self.assertEqual(isinstance(_payload, RelayServiceData), True)
        self.assertEqual(_payload.destination_str, '1')
        self.assertEqual(_payload.name, '1 R2RelaySetGroupState 4')  # + tools.bytes2str(_res.data).upper())

    def test_can_dump6(self):
        _msg = can.Message(arbitration_id=0x82084C2, is_extended_id=True, data=[0x01, 0x08, 0x0B, 0xB8],
                           extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, RelayDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.arbitration_id_ext_bits, 0x4C2)
        self.assertEqual(_res.frame, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(_res.data, bytes(b'\x01\x08\x0B\xB8'))
        self.assertEqual(_res.opcode, 0x2)
        self.assertEqual(_res.params, 0xC2)
        _payload = _res.payload
        self.assertEqual(isinstance(_payload, RelayServiceData), True)
        self.assertEqual(_payload.destination_str, '1')
        self.assertEqual(_payload.name, '1 R2RelayQueryGroupState 4')  # + tools.bytes2str(_res.data).upper())

    def test_can_dump7(self):
        _msg = can.Message(arbitration_id=0xC2027CA, is_extended_id=True, data=[0x08, 0x0B, 0xB8],
                           extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, DaliDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.arbitration_id_ext_bits, 0x7CA)
        self.assertEqual(_res.frame, bytes(b'\x08\x0B\xB8'))
        self.assertEqual(_res.is_error, True)
        self.assertEqual(_res.data, b'')
        self.assertEqual(_res.error_code, 12)
        self.assertEqual(isinstance(_res.payload, DaliResponse), True)
        self.assertEqual(_res.payload.name, 'DaliResponse error')

    def test_can_dump8(self):
        _msg = can.Message(arbitration_id=0x8201080, is_extended_id=True, data=[0x90, 0x0B, 0xC2], extended_id=False)
        _res = CanDump.can_dump(_msg)
        self.assertEqual(isinstance(_res, ModuleDump), True)
        self.assertEqual(_res.ext_mode, True)
        self.assertEqual(_res.arbitration_id_ext_bits, 0x080)
        self.assertEqual(_res.frame, bytes(b'\x90\x0B\xC2'))
        self.assertEqual(_res.data, bytes(b'\x90\x0B\xC2'))
        self.assertEqual(_res.command_byte, 0x2)
        _payload = _res.payload
        self.assertEqual(isinstance(_payload, ServicePacket), True)
        self.assertEqual(_payload.destination_str, '1')
        self.assertEqual(_payload.name, 'R2IfaceTypes 90 0B C2')


if __name__ == '__main__':
    unittest.main()
