import unittest

from resh.utils import tools
from resh.utils.errors import BadParameter


class TestTools(unittest.TestCase):

    def test_get_bit(self):
        val = 0xC209460
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 28), 0)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 27), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 26), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 21), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 15), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 12), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 10), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 6), 1)
        unittest.TestCase.assertEqual(self, tools.get_bit(val, 5), 1)

    def test_get_bits(self):
        val = 0xC209460
        unittest.TestCase.assertEqual(self, tools.get_bits(val, 28, 28), 0)
        unittest.TestCase.assertEqual(self, tools.get_bits(val, 27, 27), 1)
        unittest.TestCase.assertEqual(self, tools.get_bits(val, 26, 26), 1)
        unittest.TestCase.assertEqual(self, tools.get_bits(val, 21, 25), 1)
        unittest.TestCase.assertEqual(self, tools.get_bits(val, 13, 17), 4)
        unittest.TestCase.assertEqual(self, tools.get_bits(val,  0, 12), 0x1460)
        try:
            tools.get_bits(val, 17, 13)
        except BadParameter:
            pass
        else:
            unittest.TestCase.fail(self, 'Should be exception!')

    def test_number2mask(self):
        unittest.TestCase.assertEqual(self, tools.numbers2mask([1, 2, 4, 5, 7, 8]), 0xDB)
        unittest.TestCase.assertEqual(self, tools.numbers2mask([1, 2, 3, 4, 5, 6, 7, 8]), 0xFF)
        unittest.TestCase.assertEqual(self, tools.numbers2mask([]), 0x00)
        try:
            _v = tools.numbers2mask([1, 9, 4])
        except BadParameter:
            pass
        else:
            unittest.TestCase.fail(self, 'Should be exception!')


if __name__ == '__main__':
    unittest.main()
