""" Модуль с методами работы с конфигами шлюза. """

import json
import os
import re
from subprocess import check_output

import requests

from resh.protocols.can_utils.interface import CanConnection

SUPERVISOR_CONF_DIR = '/etc/supervisor/conf.d/'

SUPERVISOR_CONF_EXT = '.conf'

RGP_SV_CLIENT_MASK = 'rgp_client_'

PARAMETER_CAN_INTERFACE = 'canInterface'
PARAMETER_HOST = 'remoteHost'
PARAMETER_PORT = 'remotePort'
RBUS_PROTOCOL = 'rbusVersion'


def get_files(path: str, file_mask='') -> list:
    """ Список имен файлов в директории по маске """
    search_str = f'ls {path}'
    if file_mask:
        search_str += f' | grep {file_mask}'
    return [file for file in os.listdir(path) if not file_mask or file_mask in file]


def rgp_sv_client_files() -> list:
    """ Список имен конфигурационных файлов шлюзов """
    return get_files(SUPERVISOR_CONF_DIR, RGP_SV_CLIENT_MASK)


def gateways_status(gw_ids=None):
    """
    Список всех шлюзов или шлюзов по переданному списку с их состояниями.
    :return:
    {
        'gw_id1': {
            'interface': {'name': '', 'ready': ''},
            'address': {'host': '', 'port': '', 'ready': '', 'port': ''}
        },
        'gw_id2': {}
    }
    """

    def _check_gateway(host):
        try:
            # hardware gateway
            ips = check_output(['hostname', '--all-ip-addresses']).decode().split()
            conn = requests.get(f'http://{host}/conn.html')
            if ';' in conn.text and conn.text.split(';')[0] in ips:
                return True
        except requests.ConnectionError:
            pass

        # software gateway
        try:
            requests.get(f'http://{host}/rapida/')
            return True
        except:
            return False

    gateways = {}
    file_names = rgp_sv_client_files()
    for file_name in file_names:
        item_id = int(file_name.split(RGP_SV_CLIENT_MASK)[1].split(SUPERVISOR_CONF_EXT)[0])
        if gw_ids and item_id not in gw_ids:
            continue

        file_path = os.popen(f'cat {os.path.join(SUPERVISOR_CONF_DIR, file_name)} | grep ^command=').read().strip()
        file_path = re.split(r'command=.+\s+--config\s+', file_path)[1].split(' ')[0].strip()
        if not os.path.isfile(file_path):
            continue

        gateway = {}
        with open(file_path, 'r') as file:
            config = json.loads(file.read())

        _interface_name = config.get(PARAMETER_CAN_INTERFACE)
        gateway['interface'] = {
            'name': _interface_name,
            'ready': CanConnection.check_can(_interface_name) if _interface_name else False
        }

        _host = config.get(PARAMETER_HOST)
        gateway['address'] = {
            'host': _host,
            'port': config.get(PARAMETER_PORT),
            'ready': _check_gateway(_host) if _host else False,
            'protocol': config.get(RBUS_PROTOCOL, 1)
        }

        gateways[item_id] = gateway
    return gateways
